@@ -330,6 +330,10 @@ icon_box_press_cb(GtkWidget *widget, GdkEventButton *event, PidginStatusBox *box
 				|| !*path)
 			gtk_widget_set_sensitive(menu_item, FALSE);
 
+#ifdef USE_HILDON
+	box->icon_box_menu = _hildon_make_menu_thumbable(box->icon_box_menu);
+#endif /* USE_HILDON */
+
 		gtk_menu_popup(GTK_MENU(box->icon_box_menu), NULL, NULL, NULL, NULL,
 			       event->button, event->time);
 
@@ -339,6 +343,19 @@ icon_box_press_cb(GtkWidget *widget, GdkEventButton *event, PidginStatusBox *box
 	return FALSE;
 }
 
+static void
+icon_box_tap_and_hold_cb(GtkWidget *widget, PidginStatusBox *box)
+{
+	GdkEventButton *event = (GdkEventButton *)gdk_event_new(GDK_BUTTON_PRESS);
+
+	event->button = 3;
+	event->time = GDK_CURRENT_TIME;
+
+	icon_box_press_cb(widget, event, box);
+
+	gdk_event_free((GdkEvent *)event);
+}
+
 static void
 icon_box_dnd_cb(GtkWidget *widget, GdkDragContext *dc, gint x, gint y,
 		GtkSelectionData *sd, guint info, guint t, PidginStatusBox *box)
@@ -494,7 +511,13 @@ setup_icon_box(PidginStatusBox *status_box)
 	g_signal_connect(G_OBJECT(status_box->icon_box), "drag_data_received", G_CALLBACK(icon_box_dnd_cb), status_box);
 	g_signal_connect(G_OBJECT(status_box->icon_box), "enter-notify-event", G_CALLBACK(icon_box_enter_cb), status_box);
 	g_signal_connect(G_OBJECT(status_box->icon_box), "leave-notify-event", G_CALLBACK(icon_box_leave_cb), status_box);
+#ifdef USE_HILDON
+	g_signal_connect(G_OBJECT(status_box->icon_box), "button-release-event", G_CALLBACK(icon_box_press_cb), status_box);
+	gtk_widget_tap_and_hold_setup(status_box->icon_box, NULL, NULL, 0);
+	g_signal_connect(G_OBJECT(status_box->icon_box), "tap-and-hold", G_CALLBACK(icon_box_tap_and_hold_cb), status_box);
+#else /* !USE_HILDON */
 	g_signal_connect(G_OBJECT(status_box->icon_box), "button-press-event", G_CALLBACK(icon_box_press_cb), status_box);
+#endif /* USE_HILDON */
 
 	gtk_container_add(GTK_CONTAINER(status_box->icon_box), status_box->icon);
 	gtk_widget_show(status_box->icon);
@@ -2556,11 +2579,6 @@ static void update_size(PidginStatusBox *status_box)
 	gboolean interior_focus;
 	int focus_width;
 
-	gtk_widget_style_get (status_box->imhtml,
-		"interior-focus", &interior_focus,
-		"focus-line-width", &focus_width,
-		NULL);
-
 	if (!status_box->imhtml_visible)
 	{
 		if (status_box->vbox != NULL)
@@ -2609,6 +2627,11 @@ static void update_size(PidginStatusBox *status_box)
 
 	height += (pad_top + pad_bottom) * lines;
 	height += (pad_inside) * (display_lines - lines);
+
+	gtk_widget_style_get(status_box->imhtml,
+	                     "interior-focus", &interior_focus,
+	                     "focus-line-width", &focus_width,
+	                     NULL);
 	if (!interior_focus)
 		height += 2 * focus_width;
 
@@ -2642,6 +2665,7 @@ static void pidgin_status_box_changed(PidginStatusBox *status_box)
 	gpointer data;
 	GList *accounts = NULL, *node;
 	int active;
+	gboolean wastyping = FALSE;
 
 
 	if (!gtk_tree_model_get_iter (GTK_TREE_MODEL(status_box->dropdown_store), &iter, path))
@@ -2654,7 +2678,7 @@ static void pidgin_status_box_changed(PidginStatusBox *status_box)
 			   TYPE_COLUMN, &type,
 			   DATA_COLUMN, &data,
 			   -1);
-	if (status_box->typing != 0)
+	if ((wastyping = (status_box->typing != 0)))
 		purple_timeout_remove(status_box->typing);
 	status_box->typing = 0;
 
@@ -2678,14 +2702,18 @@ static void pidgin_status_box_changed(PidginStatusBox *status_box)
 			pidgin_status_editor_show(FALSE,
 				purple_savedstatus_is_transient(saved_status)
 					? saved_status : NULL);
-			status_menu_refresh_iter(status_box, FALSE);
+			status_menu_refresh_iter(status_box, wastyping);
+			if (wastyping)
+				pidgin_status_box_refresh(status_box);
 			return;
 		}
 
 		if (type == PIDGIN_STATUS_BOX_TYPE_SAVED)
 		{
 			pidgin_status_window_show();
-			status_menu_refresh_iter(status_box, FALSE);
+			status_menu_refresh_iter(status_box, wastyping);
+			if (wastyping)
+				pidgin_status_box_refresh(status_box);
 			return;
 		}
 	}
