@@ -882,6 +882,10 @@ void _hildon_file_chooser_dialog_mods(GtkWidget *file_chooser);
 
 GtkWidget *_hildon_make_menu_thumbable(GtkWidget *menu);
 
+#if HILDON_CHECK_VERSION(2,1,0)
+GtkWidget *_hildon_make_fullscreen_button(gboolean auto_hide);
+#endif /* HILDON >= 2.1.0 */
+
 #endif /* USE_HILDON */
 
 #endif /* _PIDGINUTILS_H_ */
